(*
  Copyright 2010 Gerd Stolpmann

  This file is part of Plasma, a distributed filesystem and a
  map/reduce computation framework. Unless you have a written license
  agreement with the copyright holder (Gerd Stolpmann), the following
  terms apply:

  Plasma is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Plasma is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

*)
(* $Id: nn_datanode_ctrl.mli 239 2010-06-23 16:49:03Z gerd $ *)

(** Push safetrans secrets to the datanodes *)

val reset : unit -> unit
  (** Executes a safetrans reset in the background *)

val safetrans_e : st_id:int64 -> st_tmo:int64 -> st_secret:int64 ->
                  int list Uq_engines.engine
  (** Pushes a safetrans secret to all enabled datanodes. If this is the
      first invocation, the safetrans are reset beforehand.
      Returns the list of datastore ID's to which the new safetrans
      could be pushed (which may be empty!)
   *)

val cancel_safetrans_e : st_id:int64 -> int list ->
                         unit Uq_engines.engine
  (** Pushes a safetrans cancellation to all datanodes whose ID's
      are passed in the list.
   *)

val disconnect : int -> unit
  (** Disconnects from this datanode *)

val post_start_hook : unit -> unit
