(*
  Copyright 2010 Gerd Stolpmann

  This file is part of Plasma, a distributed filesystem and a
  map/reduce computation framework. Unless you have a written license
  agreement with the copyright holder (Gerd Stolpmann), the following
  terms apply:

  Plasma is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Plasma is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

*)
(* $Id: nn_push.ml 239 2010-06-23 16:49:03Z gerd $ *)

module Nslave_proxy = Pfs_rpcapi_clnt.Make'Nameslave(Rpc_proxy.ManagedClient)

type client = Rpc_proxy.ManagedClient.mclient

let create_client conn tmo esys =
  let mclient_config =
    Rpc_proxy.ManagedClient.create_mclient_config
      ~programs:[ Pfs_rpcapi_clnt.Nameslave.V1._program ]
      ~msg_timeout:tmo
      ~msg_timeout_is_fatal:true
      ~initial_ping:true
      () in
  let mclient =
    Rpc_proxy.ManagedClient.create_mclient
      mclient_config
      conn
      esys in
  mclient

let trigger_shutdown mclient =
  Rpc_proxy.ManagedClient.trigger_shutdown mclient (fun () -> ())

let begin_transaction_e mclient cn rev =
  Plasma_util.rpc_engine
    mclient Nslave_proxy.V1.begin_transaction'async (cn,rev)

let prepare_commit_e mclient =
  Plasma_util.rpc_engine mclient Nslave_proxy.V1.prepare_commit'async ()

let commit_e mclient =
  Plasma_util.rpc_engine mclient Nslave_proxy.V1.commit'async ()

let push_e m mclient =
  match m with
    | `Datastore_upd(id,identity,size,enabled) ->
	Plasma_util.rpc_engine 
	  mclient 
	  Nslave_proxy.V1.push_datastore_upd'async
	  (id,identity,size,enabled)
    | `Datastore_del id ->
	Plasma_util.rpc_engine 
	  mclient 
	  Nslave_proxy.V1.push_datastore_del'async
	  id
    | `Revision_upd revstr ->
	Plasma_util.rpc_engine 
	  mclient 
	  Nslave_proxy.V1.push_revision_upd'async
	  revstr
    | `Blockalloc_upd(datastore,blkidx,blkmap) ->
	Plasma_util.rpc_engine 
	  mclient 
	  Nslave_proxy.V1.push_blockalloc_upd'async
	  (datastore,blkidx,blkmap)
    | `Inode_ins(id,ii) ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_inode_ins'async
	  (id,ii)
    | `Inode_upd(id,ii) ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_inode_upd'async
	  (id,ii)
    | `Inode_upd_time(id,mtime,ctime) ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_inode_upd_time'async
	  (id,mtime,ctime)
    | `Inode_del id ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_inode_del'async
	  id
    | `Inodeblocks_ins (inode,bi_list) ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_inodeblocks_ins'async
	  (inode,Array.of_list bi_list)
    | `Inodeblocks_del(inode,blkidx,len) ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_inodeblocks_del'async
	  (inode,blkidx,len)
    | `Names_ins(name,inode) ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_names_ins'async
	  (name,inode)
    | `Names_del name ->
	Plasma_util.rpc_engine
	  mclient
	  Nslave_proxy.V1.push_names_del'async
	  name


