(*
  Copyright 2010 Gerd Stolpmann

  This file is part of Plasma, a distributed filesystem and a
  map/reduce computation framework. Unless you have a written license
  agreement with the copyright holder (Gerd Stolpmann), the following
  terms apply:

  Plasma is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Plasma is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

*)
(* $Id: plasma_client_config.mli 239 2010-06-23 16:49:03Z gerd $ *)

(** Configuration of clients *)

class type client_config =
object
  method clustername : string
    (** The clustername *)

  method nn_nodes : string list
    (** The name nodes in "host:port" syntax *)
end


val parse_config_file : string -> client_config list
  (** Parses the config file and returns all included configs *)

(** The config file should look like:

    {[
      plasmafs {
        cluster {
          clustername = "cluster1";
          node { addr = "hostname:port" };
          node { addr = "hostname:port" };
          ...
        }
        ... (* more [cluster] sections possible *)
      }
    ]}

    As usual, one can also have

    {[ 
       node_list = "filename";
       port = <default_port>;
    ]}

    instead of several [node]/[addr] sections.
 *)

val get_config : ?clustername:string -> 
                 ?nn_nodes:string list ->
                 unit ->
                   client_config
  (** [get_config ()]: This is the standard way for
      clients to obtain a config object:

      - If both [clustername] and [nn_nodes] are passed, this configuration
        is taken
      - The file [~/.plasmafs] is parsed and all configurations are extracted.
      - If a [clustername] is passed, this configuration is selected (it is
        an error if this does not exist)
      - If no clustername is passed, and the environment variable
        [PLASMAFS_CLUSTER] is set, this variable selects the cluster
      - Otherwise, the first configuration from [~/.plasmafs]
        is selected.
      - Otherwise the algorithm fails.
   *)
